#include <iostream>
#include <windows.h>

#pragma comment (lib, "winmm.lib")

typedef void (*pA100CallBackFunc)(unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char*, int);

int send_start_time = 0;

int mes_count = 0;

/******************************************************************
       A100DLL  " ". 
        
******************************************************************/
bool (*pA100UIntToData)(const unsigned int arg_val, unsigned char data_arr[4]);

unsigned int (*pA100DataToUInt)(const unsigned char data_arr[4]);

bool (*pA100FloatToData)(const float arg_val, unsigned char float_point_count, unsigned char data_arr[4]);

float (*pA100DataToFloat)(const unsigned char data_arr[4]);

bool (*pA100OpenSerialPort)(const std::string &interface_name, const int &interface_speed);

int (*pA100GetSerialPortCurrentState)();

void (*pA100CloseSerialPort)();

int (*pA100SendCommand)(unsigned char receiver_address, unsigned char sender_address, unsigned char command_code, unsigned char command_param, unsigned char data_array[4], int write_timeout, int read_timeout, pA100CallBackFunc a100CallBackFunc, bool b_wait);


/******************************************************************
 
******************************************************************/
void a100CallBackFunc (unsigned char dest, unsigned char source, unsigned char code, unsigned char param, unsigned char status, unsigned char error, unsigned char data[4], int error_status)
{
	if (error_status != 0) 
	{
		std::cerr<<"\n ERROR = "<<error_status<<" time elapsed = "<<timeGetTime() - send_start_time<<" ms";
	}	
	++mes_count;
	std::cerr<<"\n   :";/**/
	std::cerr<<"\n dest = "<<(int)(dest & 127);
	std::cerr<<"\n source = "<<(int)source;
	std::cerr<<"\n code = "<<(int)code;
	std::cerr<<"\n param = "<<(int)param;
	std::cerr<<"\n status = "<<(int)status;
	std::cerr<<"\n error = "<<(int)error;
	std::cerr<<"\n UINT data = "<<pA100DataToUInt(data);
	std::cerr<<"\n FLOAT data = "<<pA100DataToFloat(data);
	std::cerr<<"\n";/**/
}

/******************************************************************
MAIN
******************************************************************/
int main()
{
	timeBeginPeriod(1);
	Sleep(1000);

	setlocale(LC_ALL, "Russian");
	
	int kvant_1 = timeGetTime();
	int kvant_2 = kvant_1;
	while (kvant_1 == kvant_2) kvant_2 = timeGetTime();
	std::cerr<<"\n KVANT = "<<kvant_2 - kvant_1<<" ms";

	HMODULE runTimeDLLHandle = LoadLibrary(TEXT("A100DLL.dll"));

	if(runTimeDLLHandle == NULL)
	{
	    MessageBox(NULL, TEXT("Failed to load the DLL"), TEXT("Error"), MB_OK );
		
		timeEndPeriod(1);
	    
		return 0 ;
	} 
	else
		{
			(FARPROC &)pA100UIntToData = GetProcAddress(runTimeDLLHandle, "a100UIntToData");

			(FARPROC &)pA100DataToUInt = GetProcAddress(runTimeDLLHandle, "a100DataToUInt");
			
			(FARPROC &)pA100FloatToData = GetProcAddress(runTimeDLLHandle, "a100FloatToData");
			
			(FARPROC &)pA100DataToFloat = GetProcAddress(runTimeDLLHandle, "a100DataToFloat");
			
			(FARPROC &)pA100OpenSerialPort = GetProcAddress(runTimeDLLHandle, "a100OpenSerialPort");
			
			(FARPROC &)pA100GetSerialPortCurrentState = GetProcAddress(runTimeDLLHandle, "a100GetSerialPortCurrentState");
			
			(FARPROC &)pA100CloseSerialPort = GetProcAddress(runTimeDLLHandle, "a100CloseSerialPort");
			
			(FARPROC &)pA100SendCommand = GetProcAddress(runTimeDLLHandle, "a100SendCommand");

			if(	( ! pA100UIntToData ) ||
				( ! pA100DataToUInt ) || 
				( ! pA100FloatToData ) || 
				( ! pA100DataToFloat ) || 
				( ! pA100OpenSerialPort ) || 
				( ! pA100GetSerialPortCurrentState ) || 
				( ! pA100CloseSerialPort ) || 
				( ! pA100SendCommand ) )
			{
				MessageBox(NULL, TEXT("A100DLL functions load error !!!"),TEXT("Error"), MB_OK );
				FreeLibrary(runTimeDLLHandle);
				timeEndPeriod(1);
				return 0 ;
			}

			unsigned char data_array[4] = {0, 0, 0, 0};
			int time_out = 100;

			int send_error;
			if (pA100OpenSerialPort("COM1", 38400))
			{		
				send_start_time = timeGetTime();
				
				std::cerr<<"\n    000  000";
				send_error = pA100SendCommand(0x01, 0x1f, 0x00, 0x00, data_array, time_out, time_out, a100CallBackFunc, true);
				
				std::cerr<<"\n     0x03  000";
				send_error = pA100SendCommand(0x01, 0x1f, 0x03, 0x00, data_array, time_out, time_out, a100CallBackFunc, true);

				std::cerr<<"\n   0x18  010 - -   ";
				send_error = pA100SendCommand(0x01, 0x1f, 0x18, 0x10, data_array, time_out, time_out, a100CallBackFunc, true);
				
				std::cerr<<"\n   0x18  012 -    ";
				send_error = pA100SendCommand(0x01, 0x1f, 0x18, 0x12, data_array, time_out, time_out, a100CallBackFunc, true);

				std::cerr<<"\n   0x18  014 -     ";
				send_error = pA100SendCommand(0x01, 0x1f, 0x18, 0x14, data_array, time_out, time_out, a100CallBackFunc, true);

				if (send_error != 0)	
				{
					std::cerr<<"\n pA100SendCommand ERROR = "<<send_error;
				}

				/*int time_counter_1;
				int time_counter_2;		
				int cur_state;

				cur_state = pA100GetSerialPortCurrentState();
				for (int i = 0; i < 5; ++i)
				{
					time_counter_1 = timeGetTime();
					time_counter_2 = time_counter_1;
					mes_count = 0;

					std::cerr<<"\n "<<i<<" ***************** \n";
					while ((time_counter_2 - time_counter_1) < 1000)
					{
						send_start_time = timeGetTime();
						
						send_error = pA100SendCommand(0x01, 0x1f, 0x04, 0x00, data_array, time_out, time_out, a100CallBackFunc, true);

						if (send_error != 0)	
						{
							std::cerr<<"\n pA100SendCommand ERROR = "<<send_error;
							break;
						}
						
						//do
						//{ 
							//Sleep(1);
							//cur_state = pA100GetSerialPortCurrentState();
						//} while ((cur_state != 1) && (cur_state != 0));						

						time_counter_2 = timeGetTime();
					}
		
					std::cerr<<"\n total time = "<<time_counter_2 - time_counter_1<<" ms";
					std::cerr<<"\n mes_count = "<<mes_count;
				}*/

			}
		
			std::cin.get();
			pA100CloseSerialPort();
			FreeLibrary(runTimeDLLHandle);
	}
	
	timeEndPeriod(1);

	return 0;
}